@@ -7,34 +7,36 @@ import re
 
 import django
 import psycopg2
+from django.apps import apps
 from django.core import exceptions
 from django.db import connection, transaction
-from django.db.models.query import QuerySet
 from django.db import models
-from django.apps import apps
+from django.db.models.query import QuerySet
 
 from django_pgviews.db import get_fields_by_name
 
-
 FIELD_SPEC_REGEX = r"^([A-Za-z_][A-Za-z0-9_]*)\." r"([A-Za-z_][A-Za-z0-9_]*)\." r"(\*|(?:[A-Za-z_][A-Za-z0-9_]*))$"
 FIELD_SPEC_RE = re.compile(FIELD_SPEC_REGEX)
 
+ViewSQL = collections.namedtuple("ViewSQL", "query,params")
+
 log = logging.getLogger("django_pgviews.view")
 
 
 def hasfield(model_cls, field_name):
-    """Like `hasattr()`, but for model fields.
+    """
+    Like `hasattr()`, but for model fields.
 
-        >>> from django.contrib.auth.models import User
-        >>> hasfield(User, 'password')
-        True
-        >>> hasfield(User, 'foobarbaz')
-        False
+    >>> from django.contrib.auth.models import User
+    >>> hasfield(User, 'password')
+    True
+    >>> hasfield(User, 'foobarbaz')
+    False
     """
     try:
         model_cls._meta.get_field(field_name)
         return True
-    except models.FieldDoesNotExist:
+    except exceptions.FieldDoesNotExist:
         return False
 
 
@@ -62,8 +64,176 @@ def realize_deferred_projections(sender, *args, **kwargs):
 models.signals.class_prepared.connect(realize_deferred_projections)
 
 
+def _schema_and_name(connection, view_name):
+    if "." in view_name:
+        return view_name.split(".", 1)
+    else:
+        try:
+            schema_name = connection.schema_name
+        except AttributeError:
+            schema_name = "public"
+
+        return schema_name, view_name
+
+
+def _create_mat_view(cursor, view_name, query, params, with_data):
+    """
+    Creates a materialized view using a specific cursor, name and definition.
+    """
+    cursor.execute(
+        "CREATE MATERIALIZED VIEW {0} AS {1} {2};".format(
+            view_name, query, "WITH DATA" if with_data else "WITH NO DATA"
+        ),
+        params,
+    )
+
+
+def _drop_mat_view(cursor, view_name):
+    """
+    Drops a materialized view using a specific cursor.
+    """
+    cursor.execute("DROP MATERIALIZED VIEW IF EXISTS {0} CASCADE;".format(view_name))
+
+
+def _concurrent_index_name(view_name, concurrent_index):
+    return view_name + "_" + "_".join([s.strip() for s in concurrent_index.split(",")]) + "_index"
+
+
+def _create_concurrent_index(cursor, view_name, concurrent_index):
+    cursor.execute(
+        "CREATE UNIQUE INDEX {index_name} ON {view_name} ({concurrent_index})".format(
+            view_name=view_name,
+            index_name=_concurrent_index_name(view_name, concurrent_index),
+            concurrent_index=concurrent_index,
+        )
+    )
+
+
+def _ensure_indexes(connection, cursor, view_cls, schema_name_log):
+    """
+    This function gets called when a materialized view is deemed not needing a re-create. That is however only a part
+    of the story, since that checks just the SQL of the view itself. The second part is the indexes.
+    This function gets the current indexes on the materialized view and reconciles them with the indexes that
+    should be in the view, dropping extra ones and creating new ones.
+    """
+    view_name = view_cls._meta.db_table
+    concurrent_index = view_cls._concurrent_index
+    indexes = view_cls._meta.indexes
+    vschema, vname = _schema_and_name(connection, view_name)
+
+    cursor.execute("SELECT indexname FROM pg_indexes WHERE tablename = %s AND schemaname = %s", [vname, vschema])
+
+    existing_indexes = set(x[0] for x in cursor.fetchall())
+    required_indexes = set(x.name for x in indexes)
+
+    if view_cls._concurrent_index is not None:
+        concurrent_index_name = _concurrent_index_name(view_name, concurrent_index)
+        required_indexes.add(concurrent_index_name)
+    else:
+        concurrent_index_name = None
+
+    for index_name in existing_indexes - required_indexes:
+        cursor.execute(f"DROP INDEX {index_name}")
+        log.info("pgview dropped index %s on view %s (%s)", index_name, view_name, schema_name_log)
+
+    for index_name in required_indexes - existing_indexes:
+        if index_name == concurrent_index_name:
+            _create_concurrent_index(cursor, view_name, concurrent_index)
+            log.info("pgview created concurrent index on view %s (%s)", view_name, schema_name_log)
+        else:
+            for index in indexes:
+                if index.name == index_name:
+                    connection.schema_editor().add_index(view_cls, index)
+                    log.info("pgview created index %s on view %s (%s)", index.name, view_name, schema_name_log)
+                    break
+
+
 @transaction.atomic()
-def create_view(connection, view_name, view_query, update=True, force=False, materialized=False, index=None):
+def create_materialized_view(connection, view_cls, check_sql_changed=False):
+    """
+    Create a materialized view on a connection.
+
+    Returns one of statuses EXISTS, UPDATED, CREATED.
+
+    If with_data = False, then the materialized view will get created without data.
+
+    If check_sql_changed = True, then the process will first check if there is a materialized view in the database
+    already with the same SQL, if there is, it will not do anything. Otherwise the materialized view gets dropped
+    and recreated.
+    """
+    view_name = view_cls._meta.db_table
+    view_query = view_cls.get_sql()
+    concurrent_index = view_cls._concurrent_index
+
+    try:
+        schema_name = connection.schema_name
+        schema_name_log = f"schema {schema_name}"
+    except AttributeError:
+        schema_name_log = "default schema"
+
+    vschema, vname = _schema_and_name(connection, view_name)
+
+    cursor_wrapper = connection.cursor()
+    cursor = cursor_wrapper.cursor
+
+    try:
+        cursor.execute(
+            "SELECT COUNT(*) FROM pg_matviews WHERE schemaname = %s and matviewname = %s;",
+            [vschema, vname],
+        )
+        view_exists = cursor.fetchone()[0] > 0
+
+        query = view_query.query.strip()
+        if query.endswith(";"):
+            query = query[:-1]
+
+        if check_sql_changed and view_exists:
+            temp_viewname = view_name + "_temp"
+            _, temp_vname = _schema_and_name(connection, temp_viewname)
+
+            _drop_mat_view(cursor, temp_viewname)
+            _create_mat_view(cursor, temp_viewname, query, view_query.params, with_data=False)
+
+            cursor.execute(
+                "SELECT definition FROM pg_matviews WHERE schemaname = %s and matviewname IN (%s, %s);",
+                [vschema, vname, temp_vname],
+            )
+            definitions = cursor.fetchall()
+
+            _drop_mat_view(cursor, temp_viewname)
+
+            if definitions[0] == definitions[1]:
+                _ensure_indexes(connection, cursor, view_cls, schema_name_log)
+                return "EXISTS"
+
+        if view_exists:
+            _drop_mat_view(cursor, view_name)
+            log.info("pgview dropped materialized view %s (%s)", view_name, schema_name_log)
+
+        _create_mat_view(cursor, view_name, query, view_query.params, with_data=view_cls.with_data)
+        log.info("pgview created materialized view %s (%s)", view_name, schema_name_log)
+
+        if concurrent_index is not None:
+            _create_concurrent_index(cursor, view_name, concurrent_index)
+            log.info("pgview created concurrent index on view %s (%s)", view_name, schema_name_log)
+
+        if view_cls._meta.indexes:
+            schema_editor = connection.schema_editor()
+
+            for index in view_cls._meta.indexes:
+                schema_editor.add_index(view_cls, index)
+                log.info("pgview created index %s on view %s (%s)", index.name, view_name, schema_name_log)
+
+        if view_exists:
+            return "UPDATED"
+
+        return "CREATED"
+    finally:
+        cursor_wrapper.close()
+
+
+@transaction.atomic()
+def create_view(connection, view_name, view_query: ViewSQL, update=True, force=False):
     """
     Create a named view on a connection.
 
@@ -76,10 +246,7 @@ def create_view(connection, view_name, view_query, update=True, force=False, mat
     the new one.
     """
 
-    if "." in view_name:
-        vschema, vname = view_name.split(".", 1)
-    else:
-        vschema, vname = "public", view_name
+    vschema, vname = _schema_and_name(connection, view_name)
 
     cursor_wrapper = connection.cursor()
     cursor = cursor_wrapper.cursor
@@ -99,27 +266,21 @@ def create_view(connection, view_name, view_query, update=True, force=False, mat
             cursor.execute("CREATE TEMPORARY VIEW check_conflict AS SELECT * FROM {0};".format(view_name))
             try:
                 with transaction.atomic():
-                    cursor.execute("CREATE OR REPLACE TEMPORARY VIEW check_conflict AS {0};".format(view_query))
+                    cursor.execute(
+                        "CREATE OR REPLACE TEMPORARY VIEW check_conflict AS {0};".format(view_query.query),
+                        view_query.params,
+                    )
             except psycopg2.ProgrammingError:
                 force_required = True
             finally:
                 cursor.execute("DROP VIEW IF EXISTS check_conflict;")
 
-        if materialized:
-            cursor.execute("DROP MATERIALIZED VIEW IF EXISTS {0} CASCADE;".format(view_name))
-            cursor.execute("CREATE MATERIALIZED VIEW {0} AS {1};".format(view_name, view_query))
-            if index is not None:
-                index_sub_name = "_".join([s.strip() for s in index.split(",")])
-                cursor.execute(
-                    "CREATE UNIQUE INDEX {0}_{1}_index ON {0} ({2})".format(view_name, index_sub_name, index)
-                )
-            ret = view_exists and "UPDATED" or "CREATED"
-        elif not force_required:
-            cursor.execute("CREATE OR REPLACE VIEW {0} AS {1};".format(view_name, view_query))
+        if not force_required:
+            cursor.execute("CREATE OR REPLACE VIEW {0} AS {1};".format(view_name, view_query.query), view_query.params)
             ret = view_exists and "UPDATED" or "CREATED"
         elif force:
             cursor.execute("DROP VIEW IF EXISTS {0} CASCADE;".format(view_name))
-            cursor.execute("CREATE VIEW {0} AS {1};".format(view_name, view_query))
+            cursor.execute("CREATE VIEW {0} AS {1};".format(view_name, view_query.query), view_query.params)
             ret = "FORCED"
         else:
             ret = "FORCE_REQUIRED"
@@ -147,7 +308,8 @@ def clear_view(connection, view_name, materialized=False):
 
 class ViewMeta(models.base.ModelBase):
     def __new__(metacls, name, bases, attrs):
-        """Deal with all of the meta attributes, removing any Django does not want
+        """
+        Deal with all of the meta attributes, removing any Django does not want
         """
         # Get attributes before Django
         dependencies = attrs.pop("dependencies", [])
@@ -198,10 +360,16 @@ else:
 
 
 class View(models.Model, metaclass=ViewMeta):
-    """Helper for exposing Postgres views as Django models.
+    """
+    Helper for exposing Postgres views as Django models.
     """
 
     _deferred = False
+    sql = None
+
+    @classmethod
+    def get_sql(cls):
+        return ViewSQL(cls.sql, None)
 
     class Meta:
         abstract = True
@@ -209,7 +377,8 @@ class View(models.Model, metaclass=ViewMeta):
 
 
 def _realise_projections(app_label, model_name):
-    """Checks whether the model has been loaded and runs
+    """
+    Checks whether the model has been loaded and runs
     realise_deferred_projections() if it has.
     """
     try:
@@ -249,8 +418,7 @@ class ReadOnlyViewManager(models.Manager):
 
 
 class ReadOnlyView(View):
-    """View which cannot be altered
-    """
+    """View which cannot be altered"""
 
     _base_manager = ReadOnlyViewManager()
     objects = ReadOnlyViewManager()
@@ -266,6 +434,8 @@ class MaterializedView(View):
     http://www.postgresql.org/docs/current/static/sql-creatematerializedview.html
     """
 
+    with_data = True
+
     @classmethod
     def refresh(self, concurrently=False):
         cursor_wrapper = connection.cursor()
@@ -284,8 +454,7 @@ class MaterializedView(View):
 
 
 class ReadOnlyMaterializedView(MaterializedView):
-    """Read-only version of the materialized view
-    """
+    """Read-only version of the materialized view"""
 
     _base_manager = ReadOnlyViewManager()
     objects = ReadOnlyViewManager()
